import { Checker, JsonTypeOf } from './types';
export declare const string: Checker<string>;
export declare const number: Checker<number>;
export declare const boolean: Checker<boolean>;
export declare const nil: Checker<null>;
export declare const unknown: Checker<unknown>;
export declare function literal<T extends string>(str: T): Checker<T>;
export declare function any<T extends Checker<unknown>>(checkers: T[]): Checker<JsonTypeOf<T>>;
export declare function nullable<T>(checker: Checker<T>): Checker<T | null>;
export declare function array<T extends Checker<unknown>>(checker: T): Checker<JsonTypeOf<T>[]>;
export declare function object<T extends {
    [key: string]: Checker<unknown> | Function;
}>(properties: T): Checker<ExpandRecursively<ObjectJsonTypeOf<T>>>;
declare type FilterOptionalPropertyName<T> = T extends `${infer U}??` ? `${FilterOptionalPropertyName<U>}?` : T extends `${infer U}?` ? U : never;
declare type FilterRequiredPropertyName<T> = T extends `${infer U}??` ? `${FilterRequiredPropertyName<U>}?` : T extends `${infer U}?` ? never : T;
declare type ObjectJsonTypeOf<T extends {
    [k: string]: Checker<unknown> | Function;
}> = {
    [K in keyof T as FilterRequiredPropertyName<K>]-?: T[K] extends Checker<unknown> ? JsonTypeOf<T[K]> : T[K] extends Function ? LazyChecker<T[K]> : never;
} & {
    [K in keyof T as FilterOptionalPropertyName<K>]+?: T[K] extends Checker<unknown> ? JsonTypeOf<T[K]> : T[K] extends Function ? LazyChecker<T[K]> : never;
};
declare type LazyChecker<T extends Function> = T extends () => Checker<infer U> ? U : never;
declare type ExpandRecursively<T> = T extends object ? T extends infer O ? {
    [K in keyof O]: ExpandRecursively<O[K]>;
} : never : T;
export declare function isOptionalProperty(propertyName: string): boolean;
export declare function unescapePropertyName(propertyName: string): string;
export {};
