var string = {
    check: function (value) {
        return typeof value === 'string';
    },
};
var number = {
    check: function (value) {
        return typeof value === 'number';
    },
};
var boolean = {
    check: function (value) {
        return typeof value === 'boolean';
    },
};
var nil = {
    check: function (value) {
        return value === null;
    },
};
var unknown = {
    check: function (value) {
        return true;
    },
};
function literal(str) {
    return {
        check: function (value) {
            return value === str;
        },
    };
}
function any(checkers) {
    return {
        check: function (value) {
            return checkers.some(function (checker) { return checker.check(value); });
        },
    };
}
function nullable(checker) {
    return {
        check: function (value) {
            return value === null || checker.check(value);
        },
    };
}
function array(checker) {
    return {
        check: function (value) {
            return Array.isArray(value) && value.every(function (it) { return checker.check(it); });
        },
    };
}
function object(properties) {
    return {
        check: function (value) {
            if (typeof value !== 'object' || value === null)
                return false;
            return Object.keys(properties).every(function (key) {
                var unescapedKey = unescapePropertyName(key);
                if (has(value, unescapedKey)) {
                    var checkerOrFunction = properties[key];
                    if (isChecker(checkerOrFunction)) {
                        return checkerOrFunction.check(value[unescapedKey]);
                    }
                    var lazyChecker = checkerOrFunction();
                    if (isChecker(lazyChecker)) {
                        return lazyChecker.check(value[unescapedKey]);
                    }
                    else {
                        return false;
                    }
                }
                else {
                    return isOptionalProperty(key);
                }
            });
        },
    };
}
function isChecker(target) {
    return (typeof target === 'object' &&
        target !== null &&
        has(target, 'check') &&
        typeof target.check === 'function');
}
function isOptionalProperty(propertyName) {
    var _a;
    return ((_a = propertyName.match(/[^?]?(?:\?\?)*(\?)?$/)) === null || _a === void 0 ? void 0 : _a[1]) === '?';
}
function unescapePropertyName(propertyName) {
    var _a, _b;
    var match = propertyName.match(/^(.*?[^?]?)((?:\?\?)*)\??$/);
    if (match === null) {
        return propertyName;
    }
    var head = (_a = match[1]) !== null && _a !== void 0 ? _a : '';
    var questionMarks = (_b = match[2].replace(/\?\?/g, '?')) !== null && _b !== void 0 ? _b : '';
    return "" + head + questionMarks;
}
// https://github.com/microsoft/TypeScript/issues/21732#issuecomment-663994772
function has(target, property) {
    return property in target;
}

var checker = /*#__PURE__*/Object.freeze({
    __proto__: null,
    string: string,
    number: number,
    boolean: boolean,
    nil: nil,
    unknown: unknown,
    literal: literal,
    any: any,
    nullable: nullable,
    array: array,
    object: object,
    isOptionalProperty: isOptionalProperty,
    unescapePropertyName: unescapePropertyName
});

export { checker as j };
